/**
	This class includes some basic astronomical transformations:

	<ul>	
	<li>convertGregorian2Julian()
	<li>convertJulian2Gregorian()
	<li>localStarTime()
	<li>transformDLe2dl()
	</ul>

	@author Michal Conos
	@version 0.0.1
*/

class BasicAstroTrans {
	/**
	Converts <b>GregorianDate</b> into Julian date (which is here represented as a <b>double</b> value)

	@param gdate <b>GregorianDate</b> object
	
	@return Julian date

	@see <b>GregorianDate</b> object
	*/
	public static double convertGregorian2Julian ( GregorianDate gdate ) {
		double m, y, a, b, jd1, jd2, M, Y, D;

		M = m = gdate.getM();
		Y = y = gdate.getY();
		D = gdate.getD();

		if ( m < 2 ) {
			y--;
			m += 12;			
		}

		a = Math.floor (y/100);
		b = 2 - a + Math.floor(a/4);

		jd1 = Math.floor (365.25*y) + Math.floor(30.6001*(m+1)) + D + 1720994.5;
		jd2 = jd1 + b;

		if ( ( Y == 1582 && M >= 10 && D >= 15 ) || ( Y >= 1582 ) ) {
			return jd2;
		} else {
			return jd1;
		}
	}


	/**
	Sorry, this function is <b>NOT</b> implemented yet.
	*/
	public static GregorianDate convertJulian2Gregorian (double JD) {
		GregorianDate gd = new GregorianDate (0.0, 0.0, 0.0);
		return gd;
		
	}


	/**
	This function returns <b>StarTime</b> object, it is calculated from Julian date, 
	longitude of observer ( for whom StarTime is computed ), local time of observer.

	@param JD Julian date
	@param longitude observer's longitude
	@param t observer's local time

	@return <b>StarTime</b> object
	
	@see <b>StarTime</b> object
	*/
	public static StarTime localStarTime ( double JD, double longitude, double t ) {
		double T3, S0, S, s;
		StarTime st = new StarTime ( 0.0, 0.0, 0.0);

		T3 = (JD - 2451545.0)/36525;
		//System.out.println ("T3="+T3);
		S0 = 6.697374558 + 2400.05133691*T3 + 0.0000258622*T3*T3 - 0.0000000017*T3*T3*T3  ;
		
		//System.out.println ("S0="+S0);
		S0 -= Math.floor (S0 / 24)*24;
		//System.out.println ("S0="+S0);
		if (S0 < 0) S0+=24;
		//System.out.println ("S0="+S0);

		//System.out.println( "t="+t);

		S = S0 + 1.0027379093*t;
		if (S > 24) S-=24;
		s = S + longitude;


		st.sets (s);
		st.setS (S);
		st.setS0 (S0);

		return st;
	}


	/**
	This function in major does a transformation from one spherical system
	into another one. 
	<p>
	We've an object which has coordinates <i>D</i>, <i>L</i> in one spherical
	system and we want to know its coordinates in another one as <i>d</i>, <i>l</i>,
	we also know the angle between theese two systems which is <i>e</i>.

	@param DLe <b>SystemDLe</b> object.

	@return <b>Systemdl</b> object.

	*/
	public static Systemdl transformDLe2dl (SystemDLe DLe) {
		double D, L, e, d, l;
		double sin_d, tan_l;
		double x,y;
		Systemdl sdl = new Systemdl (0, 0);

		double cos_e, sin_e, sin_D, cos_D, cos_L, cos_D_sin_L;

		D = DLe.getD();
		L = DLe.getL();
		e = DLe.gete();

		cos_e = Math.cos(e);
		sin_e = Math.sin(e);
		sin_D = Math.sin(D);
		cos_D = Math.cos(D);
		cos_L = Math.cos(L);
		cos_D_sin_L = cos_D * Math.sin(L);

		y  = cos_D * cos_L;
		x  =  sin_e * sin_D + cos_e * cos_D_sin_L;
		sin_d = cos_e * sin_D - sin_e * cos_D_sin_L;
		tan_l = y / x;

		d = Math.asin (sin_d);
		l = Math.atan ( tan_l );
		if ( x < 0 ) l += Math.PI;

		sdl.setd (d);
		sdl.setl (l);
		
		return sdl;
	}
}
