/**
	This class implements some oprations on points in 3-dimensional space.

	<ul>
	<li>transformPoint3Dto2D()
	<li>initTables()
	<li>rotateXY()
	<li>rotate3DXY()
	<li>rotate3DXZ()
	<li>rotate3DYZ()
	<li>transformPolartoCartesian()
	</ul>

	@author Michal Conos
	@version 0.0.5
*/

public class Trans3D {

	static double sinus_a[] = new double[360];
	static double cosinus_a[] = new double[360];

	/**
	Transforms <b>Point3D</b> into 2-dimensional <b>Point2D</b>.

	@param p <b>Point3D</b>
	@param fov field of view

	@return transformed <b>Point2D</b>
	*/
	public static Point2D transformPoint3Dto2D ( Point3D p, double fov ) {
		double x3d, y3d, z3d;
		double x2d, y2d;

		y2d = x2d = -10000;
	
		double k;

		x3d = p.getx();
		y3d = p.gety();
		z3d = p.getz();

		//System.out.println("x3d="+x3d+" y3d="+y3d+" z3d="+z3d);

		if ( z3d > 0 ) {
			k = z3d + fov;
			x2d = fov * x3d / k;
			y2d = fov * y3d / k;
		}		
		
		return new Point2D (x2d, y2d);
	}

	/**
	When we use a fast version of sinus and cosinus, we need to do some initial
	inits. The user is responsible to call this method before calling any from the
	fast-computing functions.
	*/
	public static void initTables() {
		int i;
		
		for ( i = 0; i < 360; i++ ) {
			sinus_a[ i ] = Math.sin( i*Math.PI / 180 );
			cosinus_a[ i ] = Math.cos( i*Math.PI / 180 );
		}
	}

	/**
	Rotation of <b>Point2D</b> about angle <i>a</i>.

	@param p <b>Point2D</b> to rotate
	@param a angle of rotation
	*/
	public static void rotateXY ( Point2D p, double a ) {
		double x, y, cos_a, sin_a;
			
		x = p.getx();
		y = p.gety();
		
		cos_a = Math.cos(a);
		sin_a = Math.sin(a);

		p.setx ( x * cos_a - y * sin_a );
		p.sety ( x * sin_a + y * cos_a ); 
	} 

	/** 
	Rotation of <b>Point3D</b> in XY-space about angle <i>a</i>
	Fast version.

	@param p <b>Point3D</b>
	@param a angle of rotation
	*/
	public static void rotate3DXY( Point3D p, int a ) {
		double x, y, cos_a, sin_a;

		x = p.getx();
		y = p.gety();
		
		cos_a = cosinus_a[ a ];
		sin_a = sinus_a[ a ];

		p.setx( x * cos_a - y * sin_a );
		p.sety( x * sin_a + y * cos_a ); 
	}
		
	/** 
	Rotation of <b>Point3D</b> in XZ-space about angle <i>a</i>
	Fast version.

	@param p <b>Point3D</b>
	@param a angle of rotation
	*/
	public static void rotate3DXZ( Point3D p, int a ) {
		double x, z, cos_a, sin_a;

		x = p.getx();
		z = p.getz();
		
		cos_a = cosinus_a[ a ];
		sin_a = sinus_a[ a ];

		p.setx( x * cos_a - z * sin_a );
		p.setz( x * sin_a + z * cos_a ); 
	}

	/** 
	Rotation of <b>Point3D</b> in YZ-space about angle <i>a</i>
	Fast version.

	@param p <b>Point3D</b>
	@param a angle of rotation
	*/
	public static void rotate3DYZ( Point3D p, int a ) {
		double y, z, cos_a, sin_a;

		y = p.gety();
		z = p.getz();
		
		cos_a = cosinus_a[ a ];
		sin_a = sinus_a[ a ];

		p.sety( y * cos_a - z * sin_a );
		p.setz( y * sin_a + z * cos_a ); 
	}

	/** 
	Rotation of <b>Point3D</b> in XY-space about angle <i>a</i>

	@param p <b>Point3D</b>
	@param a angle of rotation
	*/
	public static void rotate3DXY( Point3D p, double a ) {
		double x, y, cos_a, sin_a;

		x = p.getx();
		y = p.gety();
		
		cos_a = Math.cos(a);
		sin_a = Math.sin(a);

		p.setx( x * cos_a - y * sin_a );
		p.sety( x * sin_a + y * cos_a ); 
	}

	/** 
	Rotation of <b>Point3D</b> in XZ-space about angle <i>a</i>

	@param p <b>Point3D</b>
	@param a angle of rotation
	*/
	public static void rotate3DXZ( Point3D p, double a ) {
		double x, z, cos_a, sin_a;

		x = p.getx();
		z = p.getz();
		
		cos_a = Math.cos(a);
		sin_a = Math.sin(a);

		p.setx( x * cos_a - z * sin_a );
		p.setz( x * sin_a + z * cos_a ); 
	}

	/** 
	Rotation of <b>Point3D</b> in YZ-space about angle <i>a</i>

	@param p <b>Point3D</b>
	@param a angle of rotation
	*/
	public static void rotate3DYZ( Point3D p, double a ) {
		double y, z, cos_a, sin_a;

		y = p.gety();
		z = p.getz();
		
		cos_a = Math.cos(a);
		sin_a = Math.sin(a);

		p.sety( y * cos_a - z * sin_a );
		p.setz( y * sin_a + z * cos_a ); 
	}

	/**
	Transforms polar system with radius (described as <b>Systemdl</b> and <i>R</i>)
	into cartesian.

	@param sdl <b>Systemdl</b> object
	@param R radius

	@return <b>Point3D</b> - point in cartesian system.
	*/
	public static Point3D transformPolartoCartesian ( Systemdl sdl, double R ) {
		double d, l;
		double cos_d;

		d = sdl.getd();
		l = sdl.getl();

		//System.out.println ("ooo d="+d+"  l="+l);

		cos_d = Math.cos(d);
		
		return new Point3D (R*Math.sin(l)*cos_d, R*Math.sin(d), R*Math.cos(l)*cos_d);
	}
}
